#!/bin/bash
# fix filewave server and mdm server folders&files permissions & ownership
# updated by christiang@filewave.com , Oct-22-2012
# updated by christiang@filewave.com , Jun-29-2015
# updated by christiang@filewave.com , Jul-04-2016 : FW11 support
# updated by nicole.jemison@filewave.com, Feb-05-2020 : FW13.2.2 support
# updated by christiang@filewave.com , Sep-27-2020 : FW14 support
# updated by christiang@filewave.com , Oct-14-2020 : fwcld folder fix permissions
# updated by sean.holden@filewave.com, Nov-17-2020 : fix datafolder if a symlink, added array and loop for server process checking
# updated by sean.holden@filewave.com, May-25-2021 : fix case statement on running_process to allow for wc adding white space on macOS

# check root 
if [ "$(id -u)" != "0" ] ; then echo "This script must be run as root" 1>&2; exit 1; fi

# check if fwxserver running 
running_process=$(pgrep -f filewave | wc -l)
continue_message="FileWave not running, fixing permissions"
stop_message="FileWave proccess running, stop before running this script: fwcontrol server stop"

case $running_process in

	*"0")
		echo "$continue_message"
	;;
	*"1")
		if [[ "$(uname -s)" == "Linux" ]]
		then
			echo "$continue_message"
		else
			echo "$stop_message"
			exit 1
		fi
	;;
	*)
		echo "$stop_message"
		exit 1
	;;
esac

# colours ? yes please.
if test -t 1; then
    ncolors=$(tput colors)
    if test -n "$ncolors" && test $ncolors -ge 8; then
        normal="$(tput sgr0)"
        red="$(tput setaf 1)"
        green="$(tput setaf 2)"
    fi
fi

# check filewave version
version=$(/usr/local/sbin/fwxserver -V | awk \{'print $2'\})
echo "Server Version $version found"
version=$(echo $version|sed -e 's/\.//g')

#find location of data folder
# NOTE
# datafolderpath must in include a / at the end to capture files in Data Folder that is a symlink
datafolderpath="/usr/local/filewave/fwxserver/Data Folder/"
fwxserverfolderpath="/usr/local/filewave/fwxserver"

if [ -e /fwxserver/Data\ Folder/ ] ; then
# "old version found"
    datafolderpath="/fwxserver/Data Folder/"
    fwxserverfolderpath="/fwxserver"
fi

#the apache user and group are called apache on linux, but _www on mac
APACHENAME="apache"
if [[ $(uname -s) == "Darwin" ]] ; then APACHENAME="_www" ; fi
FW_HOME=/usr/local/filewave

echo "Running with $fwxserverfolderpath , Data Folder at $datafolderpath , FW_Home as $FW_HOME"

echo -n "Fixing fwxserver folder ownership recursively ... "
chown -R $APACHENAME:$APACHENAME "${datafolderpath}"
find "$fwxserverfolderpath" -maxdepth 1 -type d -exec /bin/chmod 775 {} \;
if [ -d "$datafolderpath"/backups ] ; then chown -R root:wheel "$datafolderpath"/backups ; fi
echo "  ${green}Done${normal}"

echo -n "Fixing Data Folder File Access Rights"
find "$datafolderpath" -type f -print0|xargs -0 /bin/chmod 664
echo "  ${green}Done${normal}" 
echo -n "Fixing Data Folder Subfolder Access Rights"
find "$datafolderpath" -type d -print0|xargs -0 /bin/chmod 775 
echo "  ${green}Done${normal}"

echo -n "Fixing DB folder permissions"
chown root:$APACHENAME "$fwxserverfolderpath"/DB
chmod g+w "$fwxserverfolderpath"/DB
echo "  ${green}Done${normal}"

echo -n "Fixing sqlite file permissions"
find "$fwxserverfolderpath"/DB/ -maxdepth 1 -iname "*.sqlite*" -type f -exec chown root:$APACHENAME {} \; -exec chmod 664 {} \;
echo "  ${green}Done${normal}"

echo -n "Fixing fwxserver configuration files"
chown root:wheel /usr/local/etc/fwxserver.conf 
chmod 644 /usr/local/etc/fwxserver.conf
if [ -f /usr/local/etc/fwxcodes ] ; then chown root:wheel /usr/local/etc/fwxcodes ; chmod 644 /usr/local/etc/fwxcodes ;  fi
echo "  ${green}Done${normal}"

echo -n "Fixing DB encryption keys"
chown $APACHENAME:$APACHENAME "$FW_HOME"/certs/db_symmetric_*
chmod 644 "$FW_HOME"/certs/db_symmetric_key_*.aes
chmod 600 "$FW_HOME"/certs/db_symmetric_key.aes || true
echo "  ${green}Done${normal}"

echo -n "Fixing CA Certs and keys"
chown $APACHENAME:$APACHENAME "$FW_HOME"/certs/ca_for_*
chmod 644 "$FW_HOME"/certs/ca_for_*.crt
chmod 600 "$FW_HOME"/certs/ca_for_*.key
echo "  ${green}Done${normal}"

if [ "$version" -ge "570" ]; then 
    echo -n "Postgres installation found, fixing pg_data permissions"
    chown -R postgres:daemon "$fwxserverfolderpath"/DB//pg_data
    find "$fwxserverfolderpath"/DB/pg_data -type f -print0|xargs -0 /bin/chmod 600
    find "$fwxserverfolderpath"/DB/pg_data -type d -print0|xargs -0 /bin/chmod 700
    echo "  ${green}Done${normal}"
    echo "Fixing MDM directory ownership"
    cd "$FW_HOME"
    echo -n "Fixing log,certs,ipa directory ownership & permissions"
    chown -R $APACHENAME:$APACHENAME log certs ipa fwcld
    find "$FW_HOME"/certs -maxdepth 1 -iname "postgres.*" -type f -exec chown -R postgres:daemon {} \;    
    chmod 775 log certs ipa 
    chmod 664 certs/dummy.* certs/server.*
    if [ -f certs/apn.key ] ; then chmod 600 certs/apn.key ; chmod 644 certs/apn.crt ; fi
    chmod 644 certs/dhparams.pem
    chmod 664 log/*
    echo "  ${green}Done${normal}"
    echo -n "Fixing bin & django ownership & permissions"
    chown -R root:wheel bin postgresql
    chmod a+x bin/*
    if [ -d postgresql/conf ] || [ -d postgresql/log ] ; then 
        chown postgres:daemon postgresql/conf postgresql/log
    fi		
    chown -R root:$APACHENAME python django
    echo "  ${green}Done${normal}"	
fi

if [ "$version" -ge "800" ]; then
    chown $APACHENAME:$APACHENAME "$FW_HOME"/apache/.python-eggs
    if [ $(ls "$FW_HOME"/apache/.python-eggs/|wc -l) -gt 0 ] ; then
    chmod -R 755 "$FW_HOME"/apache/.python-eggs/*
    fi
    chmod 754 "$FW_HOME"/apache/.python-eggs
    chmod +t "$FW_HOME"/apache/.python-eggs
fi

if [ "$version" -ge "1400" ]; then
    echo -n "filewave version 14+ found - fixing addtl configuration files"
    
    FW_CONFIGS=/usr/local/etc/filewave

    #logs , and essential certs
    chmod 664 "$FW_HOME"/log/*.log
    chmod 664 "$FW_HOME"/log/*.lock
    chmod 600 "$FW_HOME"/certs/server.key
    chmod 664 "$FW_HOME"/certs/server.crt

    #toplevel for config directories
    chmod 755 "$FW_CONFIGS" "$FW_CONFIGS"/*
    chown root:wheel "$FW_CONFIGS" "$FW_CONFIGS"/*

    #alertmanager
    chown -R $APACHENAME:$APACHENAME "$FW_CONFIGS"/alertmanager
    chmod 600 "$FW_CONFIGS"/alertmanager/alertmanager.yml

    #borgbackup configurations
    chown -R root:wheel "$FW_CONFIGS"/backups
    chmod 644 "$FW_CONFIGS"/backups/*

    #grafana config
    chown -R $APACHENAME:$APACHENAME "$FW_CONFIGS"/grafana
    chmod -R 755 "$FW_CONFIGS"/grafana
    chmod 600 "$FW_CONFIGS"/grafana/conf/filewave.ini

    #mtail config
    chown -R root:wheel "$FW_CONFIGS"/mtail
    find "$FW_CONFIGS"/mtail -type f -print0|xargs -0 /bin/chmod 600
    find "$FW_CONFIGS"/mtail -type d -print0|xargs -0 /bin/chmod 700

    #prometheus config
    chown -R $APACHENAME:$APACHENAME "$FW_CONFIGS"/prometheus/conf.d
    chmod -R 755 "$FW_CONFIGS"/prometheus/conf.d
    if [ -e "$FW_CONFIGS"/prometheus/conf.d/bearer_token_file ]
    then
        chmod 600 "$FW_CONFIGS"/prometheus/conf.d/bearer_token_file
    fi
    chown root:wheel "$FW_CONFIGS"/prometheus/Pipfile
    chown root:wheel "$FW_CONFIGS"/prometheus/prometheus.yml
    chmod 600 "$FW_CONFIGS"/prometheus/prometheus.yml

    #redis config
    chown -R root:wheel "$FW_CONFIGS"/redis
    chmod 644 "$FW_CONFIGS"/redis/*
    echo "  ${green}Done${normal}"
fi

echo "Done! Please start the server: fwcontrol server start"
